@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -31,7 +31,7 @@ extern "C" {
 
 #include <connman/property.h>
 #include <connman/types.h>
-#include <connman/ipv4.h>
+#include <connman/ipconfig.h>
 
 /**
  * SECTION:element
@@ -46,13 +46,9 @@ enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_DEVICE     = 3,
 	CONNMAN_ELEMENT_TYPE_NETWORK    = 4,
 	CONNMAN_ELEMENT_TYPE_SERVICE    = 5,
-	CONNMAN_ELEMENT_TYPE_PPP        = 6,
-	CONNMAN_ELEMENT_TYPE_IPV4       = 7,
 	CONNMAN_ELEMENT_TYPE_IPV6       = 8,
-	CONNMAN_ELEMENT_TYPE_DHCP       = 9,
 	CONNMAN_ELEMENT_TYPE_BOOTP      = 10,
 	CONNMAN_ELEMENT_TYPE_ZEROCONF   = 11,
-	CONNMAN_ELEMENT_TYPE_CONNECTION = 42,
 	CONNMAN_ELEMENT_TYPE_VENDOR     = 10000,
 };
 
@@ -64,9 +60,10 @@ enum connman_element_state {
 };
 
 enum connman_element_error {
-	CONNMAN_ELEMENT_ERROR_UNKNOWN     = 0,
-	CONNMAN_ELEMENT_ERROR_FAILED      = 1,
-	CONNMAN_ELEMENT_ERROR_DHCP_FAILED = 2,
+	CONNMAN_ELEMENT_ERROR_UNKNOWN        = 0,
+	CONNMAN_ELEMENT_ERROR_FAILED         = 1,
+	CONNMAN_ELEMENT_ERROR_DHCP_FAILED    = 2,
+	CONNMAN_ELEMENT_ERROR_CONNECT_FAILED = 3,
 };
 
 struct connman_driver;
@@ -80,9 +77,9 @@ struct connman_element {
 	enum connman_element_state state;
 	enum connman_element_error error;
 	gboolean enabled;
-	gboolean configuring;
 	gchar *devname;
 
+	GHashTable *children;
 	struct connman_element *parent;
 
 	struct connman_driver *driver;
@@ -98,46 +95,66 @@ struct connman_element {
 
 	GHashTable *properties;
 
+	gchar *hostname;
+	gchar *domainname;
+
 	struct {
-		enum connman_ipv4_method method;
+		enum connman_ipconfig_method method;
 		gchar *address;
+		gchar *peer;
 		gchar *netmask;
 		gchar *gateway;
 		gchar *network;
 		gchar *broadcast;
 		gchar *nameserver;
+		gchar *timeserver;
+		gchar *pac;
 	} ipv4;
+
+	struct {
+		enum connman_ipconfig_method method;
+		gchar *address;
+		int prefix_len;
+		gchar *gateway;
+		gchar *network;
+	} ipv6;
 };
 
-extern struct connman_element *connman_element_create(const char *name);
-extern struct connman_element *connman_element_ref(struct connman_element *element);
-extern void connman_element_unref(struct connman_element *element);
+struct connman_element *connman_element_create(const char *name);
+struct connman_element *connman_element_ref(struct connman_element *element);
+void connman_element_unref(struct connman_element *element);
 
-extern int connman_element_get_value(struct connman_element *element,
+int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value);
 
-extern int connman_element_set_string(struct connman_element *element,
+int connman_element_set_string(struct connman_element *element,
 					const char *key, const char *value);
-extern const char *connman_element_get_string(struct connman_element *element,
+const char *connman_element_get_string(struct connman_element *element,
 							const char *key);
-extern int connman_element_set_uint8(struct connman_element *element,
+int connman_element_set_bool(struct connman_element *element,
+				const char *key, connman_bool_t value);
+connman_bool_t connman_element_get_bool(struct connman_element *element,
+							const char *key);
+int connman_element_set_uint8(struct connman_element *element,
 				const char *key, connman_uint8_t value);
-extern connman_uint8_t connman_element_get_uint8(struct connman_element *element,
+connman_uint8_t connman_element_get_uint8(struct connman_element *element,
 							const char *key);
-extern int connman_element_set_blob(struct connman_element *element,
+int connman_element_set_blob(struct connman_element *element,
 			const char *key, const void *data, unsigned int size);
-extern const void *connman_element_get_blob(struct connman_element *element,
+const void *connman_element_get_blob(struct connman_element *element,
 					const char *key, unsigned int *size);
 
-extern int connman_element_register(struct connman_element *element,
+int connman_element_register(struct connman_element *element,
 					struct connman_element *parent);
-extern void connman_element_unregister(struct connman_element *element);
-extern void connman_element_unregister_children(struct connman_element *element);
-extern void connman_element_update(struct connman_element *element);
+void connman_element_unregister(struct connman_element *element);
+void connman_element_unregister_children(struct connman_element *element);
+void connman_element_unregister_children_type(struct connman_element *element,
+						enum connman_element_type type);
+void connman_element_update(struct connman_element *element);
 
-extern int connman_element_set_enabled(struct connman_element *element,
+int connman_element_set_enabled(struct connman_element *element,
 							gboolean enabled);
-extern void connman_element_set_error(struct connman_element *element,
+void connman_element_set_error(struct connman_element *element,
 					enum connman_element_error error);
 
 static inline void *connman_element_get_data(struct connman_element *element)
