@@ -40,14 +40,24 @@ extern "C" {
 enum connman_property_id {
 	CONNMAN_PROPERTY_ID_INVALID = 0,
 
+	CONNMAN_PROPERTY_ID_HOSTNAME,
+	CONNMAN_PROPERTY_ID_DOMAINNAME,
+
 	CONNMAN_PROPERTY_ID_IPV4_METHOD,
 	CONNMAN_PROPERTY_ID_IPV4_ADDRESS,
+	CONNMAN_PROPERTY_ID_IPV4_PEER,
 	CONNMAN_PROPERTY_ID_IPV4_NETMASK,
 	CONNMAN_PROPERTY_ID_IPV4_GATEWAY,
 	CONNMAN_PROPERTY_ID_IPV4_BROADCAST,
 	CONNMAN_PROPERTY_ID_IPV4_NAMESERVER,
 	CONNMAN_PROPERTY_ID_IPV4_TIMESERVER,
 	CONNMAN_PROPERTY_ID_IPV4_PAC,
+
+	CONNMAN_PROPERTY_ID_IPV6_METHOD,
+	CONNMAN_PROPERTY_ID_IPV6_ADDRESS,
+	CONNMAN_PROPERTY_ID_IPV6_PREFIXLEN,
+	CONNMAN_PROPERTY_ID_IPV6_GATEWAY,
+	CONNMAN_PROPERTY_ID_IPV6_NAMESERVER,
 };
 
 enum connman_property_type {
