@@ -95,8 +95,11 @@ struct modem_data {
 struct network_info {
 	struct connman_network *network;
 
-	enum connman_ipconfig_method method;
-	struct connman_ipaddress ipaddress;
+	enum connman_ipconfig_method ipv4_method;
+	struct connman_ipaddress ipv4_address;
+
+	enum connman_ipconfig_method ipv6_method;
+	struct connman_ipaddress ipv6_address;
 };
 
 static int modem_probe(struct connman_device *device)
@@ -415,7 +418,7 @@ static void set_connected(struct network_info *info,
 {
 	DBG("network %p connected %d", info->network, connected);
 
-	switch (info->method) {
+	switch (info->ipv4_method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
@@ -423,13 +426,33 @@ static void set_connected(struct network_info *info,
 		return;
 
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
-		connman_network_set_ipv4_method(info->network, info->method);
-		connman_network_set_ipaddress(info->network, &info->ipaddress);
+		connman_network_set_ipv4_method(info->network,
+							info->ipv4_method);
+		connman_network_set_ipaddress(info->network,
+							&info->ipv4_address);
 
 		break;
 
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		connman_network_set_ipv4_method(info->network, info->method);
+		connman_network_set_ipv4_method(info->network,
+							info->ipv4_method);
+
+		break;
+	}
+
+	switch (info->ipv6_method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		return;
+
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		connman_network_set_ipv6_method(info->network,
+							info->ipv6_method);
+		connman_network_set_ipaddress(info->network,
+							&info->ipv6_address);
 
 		break;
 	}
@@ -563,8 +586,217 @@ static struct connman_network_driver network_driver = {
 	.disconnect	= network_disconnect,
 };
 
-static void update_settings(DBusMessageIter *array,
-				struct network_info *info);
+static void get_dns(DBusMessageIter *array, struct network_info *info)
+{
+	DBusMessageIter entry;
+	gchar *nameservers = NULL, *nameservers_old = NULL;
+
+	DBG("");
+
+
+	dbus_message_iter_recurse(array, &entry);
+
+	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
+		const char *dns;
+
+		dbus_message_iter_get_basic(&entry, &dns);
+
+		DBG("dns %s", dns);
+
+		if (nameservers == NULL) {
+
+			nameservers = g_strdup(dns);
+		} else {
+
+			nameservers_old = nameservers;
+			nameservers = g_strdup_printf("%s %s",
+						nameservers_old, dns);
+			g_free(nameservers_old);
+		}
+
+		dbus_message_iter_next(&entry);
+	}
+
+	connman_network_set_nameservers(info->network, nameservers);
+
+	g_free(nameservers);
+}
+
+static void update_ipv4_settings(DBusMessageIter *array,
+				struct network_info *info)
+{
+	DBusMessageIter dict;
+	char *address = NULL, *netmask = NULL, *gateway = NULL;
+	const char *interface = NULL;
+
+	DBG("network %p", info->network);
+
+	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
+		return;
+
+	dbus_message_iter_recurse(array, &dict);
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key, *val;
+
+		dbus_message_iter_recurse(&dict, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		DBG("key %s", key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		if (g_str_equal(key, "Interface") == TRUE) {
+			int index;
+
+			dbus_message_iter_get_basic(&value, &interface);
+
+			DBG("interface %s", interface);
+
+			index = connman_inet_ifindex(interface);
+			if (index >= 0) {
+				connman_network_set_index(info->network, index);
+			} else {
+				connman_error("Can not find interface %s",
+								interface);
+				break;
+			}
+		} else if (g_str_equal(key, "Method") == TRUE) {
+			const char *method;
+
+			dbus_message_iter_get_basic(&value, &method);
+
+			if (g_strcmp0(method, "static") == 0) {
+
+				info->ipv4_method =
+					CONNMAN_IPCONFIG_METHOD_FIXED;
+			} else if (g_strcmp0(method, "dhcp") == 0) {
+
+				info->ipv4_method =
+					CONNMAN_IPCONFIG_METHOD_DHCP;
+				break;
+			}
+		} else if (g_str_equal(key, "Address") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			address = g_strdup(val);
+
+			DBG("address %s", address);
+		} else if (g_str_equal(key, "Netmask") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			netmask = g_strdup(val);
+
+			DBG("netmask %s", netmask);
+		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
+
+			get_dns(&value, info);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			gateway = g_strdup(val);
+
+			DBG("gateway %s", gateway);
+		}
+
+		dbus_message_iter_next(&dict);
+	}
+
+
+	if (info->ipv4_method == CONNMAN_IPCONFIG_METHOD_FIXED) {
+		connman_ipaddress_set_ipv4(&info->ipv4_address, address,
+						netmask, gateway);
+	}
+
+	/* deactive, oFono send NULL inteface before deactive signal */
+	if (interface == NULL)
+		connman_network_set_index(info->network, -1);
+
+	g_free(address);
+	g_free(netmask);
+	g_free(gateway);
+}
+
+static void update_ipv6_settings(DBusMessageIter *array,
+				struct network_info *info)
+{
+	DBusMessageIter dict;
+	char *address = NULL, *gateway = NULL;
+	unsigned char prefix_length;
+	const char *interface = NULL;
+
+	DBG("network %p", info->network);
+
+	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
+		return;
+
+	dbus_message_iter_recurse(array, &dict);
+
+	info->ipv6_method = CONNMAN_IPCONFIG_METHOD_FIXED;
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key, *val;
+
+		dbus_message_iter_recurse(&dict, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		DBG("key %s", key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		if (g_str_equal(key, "Interface") == TRUE) {
+			int index;
+
+			dbus_message_iter_get_basic(&value, &interface);
+
+			DBG("interface %s", interface);
+
+			index = connman_inet_ifindex(interface);
+			if (index >= 0) {
+				connman_network_set_index(info->network, index);
+			} else {
+				connman_error("Can not find interface %s",
+								interface);
+				break;
+			}
+		} else if (g_str_equal(key, "Address") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			address = g_strdup(val);
+
+			DBG("address %s", address);
+		} else if (g_str_equal(key, "PrefixLength") == TRUE) {
+			dbus_message_iter_get_basic(&value, &prefix_length);
+
+			DBG("prefix length %d", prefix_length);
+		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
+
+			get_dns(&value, info);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			gateway = g_strdup(val);
+
+			DBG("gateway %s", gateway);
+		}
+
+		dbus_message_iter_next(&dict);
+	}
+
+	connman_ipaddress_set_ipv6(&info->ipv6_address, address,
+						prefix_length, gateway);
+
+	/* deactive, oFono send NULL inteface before deactive signal */
+	if (interface == NULL)
+		connman_network_set_index(info->network, -1);
+
+	g_free(address);
+	g_free(gateway);
+}
 
 static int add_network(struct connman_device *device,
 			const char *path, DBusMessageIter *dict)
@@ -604,7 +836,8 @@ static int add_network(struct connman_device *device,
 		return -ENOMEM;
 	}
 
-	connman_ipaddress_clear(&info->ipaddress);
+	connman_ipaddress_clear(&info->ipv4_address);
+	connman_ipaddress_clear(&info->ipv6_address);
 	info->network = network;
 
 	connman_network_set_string(network, "Path", path);
@@ -651,7 +884,9 @@ static int add_network(struct connman_device *device,
 				goto error;
 			}
 		} else if (g_str_equal(key, "Settings"))
-			update_settings(&value, info);
+			update_ipv4_settings(&value, info);
+		else if (g_str_equal(key, "IPv6.Settings"))
+			update_ipv6_settings(&value, info);
 		else if (g_str_equal(key, "Active") == TRUE)
 			dbus_message_iter_get_basic(&value, &active);
 
@@ -1582,138 +1817,6 @@ static gboolean modem_removed(DBusConnection *connection,
 	return TRUE;
 }
 
-
-static void get_dns(DBusMessageIter *array, struct network_info *info)
-{
-	DBusMessageIter entry;
-	gchar *nameservers = NULL, *nameservers_old = NULL;
-
-	DBG("");
-
-
-	dbus_message_iter_recurse(array, &entry);
-
-	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
-		const char *dns;
-
-		dbus_message_iter_get_basic(&entry, &dns);
-
-		DBG("dns %s", dns);
-
-		if (nameservers == NULL) {
-
-			nameservers = g_strdup(dns);
-		} else {
-
-			nameservers_old = nameservers;
-			nameservers = g_strdup_printf("%s %s",
-						nameservers_old, dns);
-			g_free(nameservers_old);
-		}
-
-		dbus_message_iter_next(&entry);
-	}
-
-	connman_network_set_nameservers(info->network, nameservers);
-
-	g_free(nameservers);
-}
-
-static void update_settings(DBusMessageIter *array,
-				struct network_info *info)
-{
-	DBusMessageIter dict;
-	char *address = NULL, *netmask = NULL, *gateway = NULL;
-	const char *interface = NULL;
-
-	DBG("network %p", info->network);
-
-	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
-		return;
-
-	dbus_message_iter_recurse(array, &dict);
-
-	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key, *val;
-
-		dbus_message_iter_recurse(&dict, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		DBG("key %s", key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		if (g_str_equal(key, "Interface") == TRUE) {
-			int index;
-
-			dbus_message_iter_get_basic(&value, &interface);
-
-			DBG("interface %s", interface);
-
-			index = connman_inet_ifindex(interface);
-			if (index >= 0) {
-				connman_network_set_index(info->network, index);
-			} else {
-				connman_error("Can not find interface %s",
-								interface);
-				break;
-			}
-		} else if (g_str_equal(key, "Method") == TRUE) {
-			const char *method;
-
-			dbus_message_iter_get_basic(&value, &method);
-
-			if (g_strcmp0(method, "static") == 0) {
-
-				info->method = CONNMAN_IPCONFIG_METHOD_FIXED;
-			} else if (g_strcmp0(method, "dhcp") == 0) {
-
-				info->method = CONNMAN_IPCONFIG_METHOD_DHCP;
-				break;
-			}
-		} else if (g_str_equal(key, "Address") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			address = g_strdup(val);
-
-			DBG("address %s", address);
-		} else if (g_str_equal(key, "Netmask") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			netmask = g_strdup(val);
-
-			DBG("netmask %s", netmask);
-		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
-
-			get_dns(&value, info);
-		} else if (g_str_equal(key, "Gateway") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			gateway = g_strdup(val);
-
-			DBG("gateway %s", gateway);
-		}
-
-		dbus_message_iter_next(&dict);
-	}
-
-
-	if (info->method == CONNMAN_IPCONFIG_METHOD_FIXED) {
-		connman_ipaddress_set_ipv4(&info->ipaddress, address,
-						netmask, gateway);
-	}
-
-	/* deactive, oFono send NULL inteface before deactive signal */
-	if (interface == NULL)
-		connman_network_set_index(info->network, -1);
-
-	g_free(address);
-	g_free(netmask);
-	g_free(gateway);
-}
-
 static gboolean context_changed(DBusConnection *connection,
 					DBusMessage *message, void *user_data)
 {
@@ -1744,7 +1847,9 @@ static gboolean context_changed(DBusConnection *connection,
 	DBG("key %s", key);
 
 	if (g_str_equal(key, "Settings") == TRUE)
-		update_settings(&value, info);
+		update_ipv4_settings(&value, info);
+	else if (g_str_equal(key, "IPv6.Settings"))
+			update_ipv6_settings(&value, info);
 	else if (g_str_equal(key, "Active") == TRUE) {
 		dbus_bool_t active;
 
