@@ -54,6 +54,9 @@ DBusMessage *__connman_error_invalid_property(DBusMessage *msg);
 int __connman_manager_init(void);
 void __connman_manager_cleanup(void);
 
+int __connman_clock_init(void);
+void __connman_clock_cleanup(void);
+
 int __connman_agent_init(void);
 void __connman_agent_cleanup(void);
 
@@ -72,7 +75,8 @@ void __connman_counter_cleanup(void);
 struct connman_service *service;
 
 typedef void (* passphrase_cb_t) (struct connman_service *service,
-				const char *passphrase, void *user_data);
+				const char *identity, const char *passphrase,
+				void *user_data);
 typedef void (* report_error_cb_t) (struct connman_service *service,
 				gboolean retry, void *user_data);
 int __connman_agent_request_input(struct connman_service *service,
@@ -92,6 +96,8 @@ void __connman_debug_list_enabled(DBusMessageIter *iter, void *user_data);
 
 #include <connman/option.h>
 
+#include <connman/setting.h>
+
 #include <connman/plugin.h>
 
 int __connman_plugin_init(const char *pattern, const char *exclude);
@@ -110,6 +116,15 @@ int __connman_inet_modify_address(int cmd, int flags, int index, int family,
 				unsigned char prefixlen,
 				const char *broadcast);
 
+#include <netinet/ip6.h>
+#include <netinet/icmp6.h>
+
+typedef void (*__connman_inet_rs_cb_t) (struct nd_router_advert *reply,
+					void *user_data);
+
+int __connman_inet_ipv6_send_rs(int index, int timeout,
+			__connman_inet_rs_cb_t callback, void *user_data);
+
 #include <connman/wifi.h>
 
 #include <connman/rfkill.h>
@@ -247,7 +262,8 @@ void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter);
 void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
-							DBusMessageIter *iter);
+					DBusMessageIter *iter,
+					struct connman_ipconfig *ip4config);
 void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter);
 int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
@@ -403,6 +419,7 @@ connman_bool_t __connman_network_get_weakness(struct connman_network *network);
 int __connman_config_init();
 void __connman_config_cleanup(void);
 
+int __connman_config_load_service(GKeyFile *keyfile, const char *group, connman_bool_t persistent);
 int __connman_config_provision_service(struct connman_service *service);
 
 #include <connman/profile.h>
@@ -423,13 +440,6 @@ int __connman_profile_remove(const char *path);
 
 void __connman_profile_changed(gboolean delayed);
 
-int __connman_profile_add_device(struct connman_device *device);
-int __connman_profile_remove_device(struct connman_device *device);
-
-int __connman_profile_add_network(struct connman_network *network);
-int __connman_profile_update_network(struct connman_network *network);
-int __connman_profile_remove_network(struct connman_network *network);
-
 int __connman_tethering_init(void);
 void __connman_tethering_cleanup(void);
 
@@ -487,9 +497,11 @@ const char *__connman_service_get_path(struct connman_service *service);
 unsigned int __connman_service_get_order(struct connman_service *service);
 struct connman_network *__connman_service_get_network(struct connman_service *service);
 enum connman_service_security __connman_service_get_security(struct connman_service *service);
+const char *__connman_service_get_phase2(struct connman_service *service);
 connman_bool_t __connman_service_wps_enabled(struct connman_service *service);
 int __connman_service_set_favorite(struct connman_service *service,
 						connman_bool_t favorite);
+connman_bool_t __connman_service_get_immutable(struct connman_service *service);
 int __connman_service_set_immutable(struct connman_service *service,
 						connman_bool_t immutable);
 
@@ -507,9 +519,10 @@ int __connman_service_request_login(struct connman_service *service);
 int __connman_service_lookup(const char *pattern, const char **path);
 int __connman_service_connect(struct connman_service *service);
 int __connman_service_disconnect(struct connman_service *service);
+int __connman_service_disconnect_all(void);
 int __connman_service_create_and_connect(DBusMessage *msg);
+int __connman_service_provision(DBusMessage *msg);
 void __connman_service_auto_connect(void);
-struct connman_service *__connman_service_connect_type(enum connman_service_type type);
 
 const char *__connman_service_type2string(enum connman_service_type type);
 
@@ -535,6 +548,8 @@ const char *__connman_service_get_nameserver(struct connman_service *service);
 void __connman_service_set_proxy_autoconfig(struct connman_service *service,
 							const char *url);
 
+void __connman_service_set_identity(struct connman_service *service,
+					const char *identity);
 void __connman_service_set_passphrase(struct connman_service *service,
 					const char* passphrase);
 
@@ -547,6 +562,18 @@ void __connman_service_notify(struct connman_service *service,
 int __connman_service_counter_register(const char *counter);
 void __connman_service_counter_unregister(const char *counter);
 
+struct connman_session;
+typedef connman_bool_t (* service_match_cb) (struct connman_session *session,
+					struct connman_service *service);
+
+GSequence *__connman_service_get_list(struct connman_session *session,
+					service_match_cb service_match);
+
+connman_bool_t __connman_service_is_connecting(struct connman_service *service);
+connman_bool_t __connman_service_is_connected(struct connman_service *service);
+connman_bool_t __connman_service_is_idle(struct connman_service *service);
+const char *__connman_service_get_name(struct connman_service *service);
+
 #include <connman/location.h>
 
 int __connman_location_init(void);
@@ -572,6 +599,8 @@ void __connman_notifier_list_connected(DBusMessageIter *iter, void *user_data);
 
 void __connman_notifier_register(enum connman_service_type type);
 void __connman_notifier_unregister(enum connman_service_type type);
+void __connman_notifier_service_add(struct connman_service *service);
+void __connman_notifier_service_remove(struct connman_service *service);
 void __connman_notifier_enable(enum connman_service_type type);
 void __connman_notifier_disable(enum connman_service_type type);
 void __connman_notifier_connect(enum connman_service_type type);
@@ -579,6 +608,10 @@ void __connman_notifier_disconnect(enum connman_service_type type);
 void __connman_notifier_offlinemode(connman_bool_t enabled);
 void __connman_notifier_default_changed(struct connman_service *service);
 void __connman_notifier_proxy_changed(struct connman_service *service);
+void __connman_notifier_service_state_changed(struct connman_service *service,
+					enum connman_service_state state);
+void __connman_notifier_ipconfig_changed(struct connman_service *service,
+					struct connman_ipconfig *ipconfig);
 
 connman_bool_t __connman_notifier_is_registered(enum connman_service_type type);
 connman_bool_t __connman_notifier_is_enabled(enum connman_service_type type);
@@ -597,8 +630,12 @@ unsigned int __connman_rtnl_update_interval_remove(unsigned int interval);
 int __connman_rtnl_request_update(void);
 int __connman_rtnl_send(const void *buf, size_t len);
 
-int __connman_session_release(const char *owner);
-struct connman_service *__connman_session_request(const char *bearer, const char *owner);
+connman_bool_t __connman_session_mode();
+void __connman_session_set_mode(connman_bool_t enable);
+
+int __connman_session_create(DBusMessage *msg);
+int __connman_session_destroy(DBusMessage *msg);
+
 int __connman_session_init(void);
 void __connman_session_cleanup(void);
 
@@ -636,3 +673,7 @@ void __connman_dnsproxy_cleanup(void);
 int __connman_dnsproxy_append(const char *interface, const char *domain, const char *server);
 int __connman_dnsproxy_remove(const char *interface, const char *domain, const char *server);
 void __connman_dnsproxy_flush(void);
+
+int __connman_6to4_probe(struct connman_service *service);
+void __connman_6to4_remove(struct connman_ipconfig *ipconfig);
+int __connman_6to4_check(struct connman_ipconfig *ipconfig);
