@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
 #include <net/if.h>
 #include <net/if_arp.h>
 #include <linux/if_link.h>
@@ -50,6 +51,8 @@ struct connman_ipconfig {
 	enum connman_ipconfig_method method;
 	struct connman_ipaddress *address;
 	struct connman_ipaddress *system;
+
+	int ipv6_privacy_config;
 };
 
 struct connman_ipdevice {
@@ -76,6 +79,9 @@ struct connman_ipdevice {
 
 	struct connman_ipconfig *config_ipv4;
 	struct connman_ipconfig *config_ipv6;
+
+	gboolean ipv6_enabled;
+	int ipv6_privacy;
 };
 
 static GHashTable *ipdevice_hash = NULL;
@@ -139,6 +145,9 @@ static gboolean check_ipv6_address(const char *address)
 	unsigned char buf[sizeof(struct in6_addr)];
 	int err;
 
+	if (address == NULL)
+		return FALSE;
+
 	err = inet_pton(AF_INET6, address, buf);
 	if (err > 0)
 		return TRUE;
@@ -147,8 +156,9 @@ static gboolean check_ipv6_address(const char *address)
 }
 
 int connman_ipaddress_set_ipv6(struct connman_ipaddress *ipaddress,
-				const char *address, const char *gateway,
-						unsigned char prefix_length)
+				const char *address,
+				unsigned char prefix_length,
+				const char *gateway)
 {
 	if (ipaddress == NULL)
 		return -EINVAL;
@@ -162,6 +172,8 @@ int connman_ipaddress_set_ipv6(struct connman_ipaddress *ipaddress,
 	DBG("prefix_len %d address %s gateway %s",
 			prefix_length, address, gateway);
 
+	ipaddress->family = AF_INET6;
+
 	ipaddress->prefixlen = prefix_length;
 
 	g_free(ipaddress->local);
@@ -173,11 +185,13 @@ int connman_ipaddress_set_ipv6(struct connman_ipaddress *ipaddress,
 	return 0;
 }
 
-void connman_ipaddress_set_ipv4(struct connman_ipaddress *ipaddress,
+int connman_ipaddress_set_ipv4(struct connman_ipaddress *ipaddress,
 		const char *address, const char *netmask, const char *gateway)
 {
 	if (ipaddress == NULL)
-		return;
+		return -EINVAL;
+
+	ipaddress->family = AF_INET;
 
 	ipaddress->prefixlen = __connman_ipconfig_netmask_prefix_len(netmask);
 
@@ -186,6 +200,18 @@ void connman_ipaddress_set_ipv4(struct connman_ipaddress *ipaddress,
 
 	g_free(ipaddress->gateway);
 	ipaddress->gateway = g_strdup(gateway);
+
+	return 0;
+}
+
+void connman_ipaddress_set_peer(struct connman_ipaddress *ipaddress,
+				const char *peer)
+{
+	if (ipaddress == NULL)
+		return;
+
+	g_free(ipaddress->peer);
+	ipaddress->peer = g_strdup(peer);
 }
 
 void connman_ipaddress_clear(struct connman_ipaddress *ipaddress)
@@ -291,6 +317,125 @@ static const char *scope2str(unsigned char scope)
 	return "";
 }
 
+static gboolean get_ipv6_state(gchar *ifname)
+{
+	int disabled;
+	gchar *path;
+	FILE *f;
+	gboolean enabled = FALSE;
+
+	if (ifname == NULL)
+		path = g_strdup("/proc/sys/net/ipv6/conf/all/disable_ipv6");
+	else
+		path = g_strdup_printf(
+			"/proc/sys/net/ipv6/conf/%s/disable_ipv6", ifname);
+
+	if (path == NULL)
+		return enabled;
+
+	f = fopen(path, "r");
+
+	g_free(path);
+
+	if (f != NULL) {
+		if (fscanf(f, "%d", &disabled) > 0)
+			enabled = !disabled;
+		fclose(f);
+	}
+
+	return enabled;
+}
+
+static void set_ipv6_state(gchar *ifname, gboolean enable)
+{
+	gchar *path;
+	FILE *f;
+
+	if (ifname == NULL)
+		path = g_strdup("/proc/sys/net/ipv6/conf/all/disable_ipv6");
+	else
+		path = g_strdup_printf(
+			"/proc/sys/net/ipv6/conf/%s/disable_ipv6", ifname);
+
+	if (path == NULL)
+		return;
+
+	f = fopen(path, "r+");
+
+	g_free(path);
+
+	if (f == NULL)
+		return;
+
+	if (enable == FALSE)
+		fprintf(f, "1");
+	else
+		fprintf(f, "0");
+
+	fclose(f);
+}
+
+static int get_ipv6_privacy(gchar *ifname)
+{
+	gchar *path;
+	FILE *f;
+	int value;
+
+	if (ifname == NULL)
+		return 0;
+
+	path = g_strdup_printf("/proc/sys/net/ipv6/conf/%s/use_tempaddr",
+								ifname);
+
+	if (path == NULL)
+		return 0;
+
+	f = fopen(path, "r");
+
+	g_free(path);
+
+	if (f == NULL)
+		return 0;
+
+	if (fscanf(f, "%d", &value) <= 0)
+		value = 0;
+
+	fclose(f);
+
+	return value;
+}
+
+/* Enable the IPv6 privacy extension for stateless address autoconfiguration.
+ * The privacy extension is described in RFC 3041 and RFC 4941
+ */
+static void set_ipv6_privacy(gchar *ifname, int value)
+{
+	gchar *path;
+	FILE *f;
+
+	if (ifname == NULL)
+		return;
+
+	path = g_strdup_printf("/proc/sys/net/ipv6/conf/%s/use_tempaddr",
+								ifname);
+
+	if (path == NULL)
+		return;
+
+	if (value < 0)
+		value = 0;
+
+	f = fopen(path, "r+");
+
+	g_free(path);
+
+	if (f == NULL)
+		return;
+
+	fprintf(f, "%d", value);
+	fclose(f);
+}
+
 static void free_ipdevice(gpointer data)
 {
 	struct connman_ipdevice *ipdevice = data;
@@ -314,6 +459,10 @@ static void free_ipdevice(gpointer data)
 	g_free(ipdevice->pac);
 
 	g_free(ipdevice->address);
+
+	set_ipv6_state(ipdevice->ifname, ipdevice->ipv6_enabled);
+	set_ipv6_privacy(ipdevice->ifname, ipdevice->ipv6_privacy);
+
 	g_free(ipdevice->ifname);
 	g_free(ipdevice);
 }
@@ -409,6 +558,9 @@ void __connman_ipconfig_newlink(int index, unsigned short type,
 	ipdevice->ifname = connman_inet_ifname(index);
 	ipdevice->type = type;
 
+	ipdevice->ipv6_enabled = get_ipv6_state(ipdevice->ifname);
+	ipdevice->ipv6_privacy = get_ipv6_privacy(ipdevice->ifname);
+
 	ipdevice->address = g_strdup(address);
 
 	g_hash_table_insert(ipdevice_hash, GINT_TO_POINTER(index), ipdevice);
@@ -616,7 +768,8 @@ void __connman_ipconfig_deladdr(int index, int family, const char *label,
 	ipdevice->address_list = g_slist_remove(ipdevice->address_list,
 								ipaddress);
 
-	connman_ipaddress_free(ipaddress);
+	connman_ipaddress_clear(ipaddress);
+	g_free(ipaddress);
 
 	connman_info("%s {del} address %s/%u label %s", ipdevice->ifname,
 						address, prefixlen, label);
@@ -653,7 +806,8 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 	if (ipdevice == NULL)
 		return;
 
-	if (scope == 0 && g_strcmp0(dst, "0.0.0.0") == 0) {
+	if (scope == 0 && (g_strcmp0(dst, "0.0.0.0") == 0 ||
+						g_strcmp0(dst, "::") == 0)) {
 		GSList *list;
 		GList *config_list;
 
@@ -667,7 +821,7 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 				ipdevice->config_ipv6->system->gateway =
 					g_strdup(gateway);
 			}
-		} else {
+		} else if (family == AF_INET) {
 			g_free(ipdevice->ipv4_gateway);
 			ipdevice->ipv4_gateway = g_strdup(gateway);
 
@@ -677,7 +831,8 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 				ipdevice->config_ipv4->system->gateway =
 					g_strdup(gateway);
 			}
-		}
+		} else
+			return;
 
 		for (list = ipdevice->address_list; list; list = list->next) {
 			struct connman_ipaddress *ipaddress = list->data;
@@ -717,7 +872,8 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 	if (ipdevice == NULL)
 		return;
 
-	if (scope == 0 && g_strcmp0(dst, "0.0.0.0") == 0) {
+	if (scope == 0 && (g_strcmp0(dst, "0.0.0.0") == 0 ||
+						g_strcmp0(dst, "::") == 0)) {
 		GSList *list;
 		GList *config_list;
 
@@ -730,7 +886,7 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 				g_free(ipdevice->config_ipv6->system->gateway);
 				ipdevice->config_ipv6->system->gateway = NULL;
 			}
-		} else {
+		} else if (family == AF_INET) {
 			g_free(ipdevice->ipv4_gateway);
 			ipdevice->ipv4_gateway = NULL;
 
@@ -739,7 +895,8 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 				g_free(ipdevice->config_ipv4->system->gateway);
 				ipdevice->config_ipv4->system->gateway = NULL;
 			}
-		}
+		} else
+			return;
 
 		for (list = ipdevice->address_list; list; list = list->next) {
 			struct connman_ipaddress *ipaddress = list->data;
@@ -786,7 +943,13 @@ void __connman_ipconfig_foreach(void (*function) (int index, void *user_data),
 	g_list_free(keys);
 }
 
-unsigned short __connman_ipconfig_get_type(int index)
+enum connman_ipconfig_type __connman_ipconfig_get_config_type(
+					struct connman_ipconfig *ipconfig)
+{
+	return ipconfig ? ipconfig->type : CONNMAN_IPCONFIG_TYPE_UNKNOWN;
+}
+
+unsigned short __connman_ipconfig_get_type_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -797,7 +960,7 @@ unsigned short __connman_ipconfig_get_type(int index)
 	return ipdevice->type;
 }
 
-unsigned int __connman_ipconfig_get_flags(int index)
+unsigned int __connman_ipconfig_get_flags_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -808,7 +971,7 @@ unsigned int __connman_ipconfig_get_flags(int index)
 	return ipdevice->flags;
 }
 
-const char *__connman_ipconfig_get_gateway(int index)
+const char *__connman_ipconfig_get_gateway_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -838,6 +1001,130 @@ void __connman_ipconfig_set_index(struct connman_ipconfig *ipconfig, int index)
 	ipconfig->index = index;
 }
 
+const char *__connman_ipconfig_get_local(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->local;
+}
+
+void __connman_ipconfig_set_local(struct connman_ipconfig *ipconfig, const char *address)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->local);
+	ipconfig->address->local = g_strdup(address);
+}
+
+const char *__connman_ipconfig_get_peer(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->peer;
+}
+
+void __connman_ipconfig_set_peer(struct connman_ipconfig *ipconfig, const char *address)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->peer);
+	ipconfig->address->peer = g_strdup(address);
+}
+
+const char *__connman_ipconfig_get_broadcast(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->broadcast;
+}
+
+void __connman_ipconfig_set_broadcast(struct connman_ipconfig *ipconfig, const char *broadcast)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->broadcast);
+	ipconfig->address->broadcast = g_strdup(broadcast);
+}
+
+const char *__connman_ipconfig_get_gateway(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->gateway;
+}
+
+void __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig, const char *gateway)
+{
+	DBG("");
+
+	if (ipconfig->address == NULL)
+		return;
+	g_free(ipconfig->address->gateway);
+	ipconfig->address->gateway = g_strdup(gateway);
+}
+
+int __connman_ipconfig_gateway_add(struct connman_ipconfig *ipconfig)
+{
+	struct connman_service *service;
+
+	DBG("");
+
+	if (ipconfig->address == NULL)
+		return -EINVAL;
+
+	service = __connman_service_lookup_from_index(ipconfig->index);
+	if (service == NULL)
+		return -EINVAL;
+
+	__connman_connection_gateway_remove(service);
+
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+		return __connman_connection_gateway_add(service, NULL,
+						ipconfig->address->gateway,
+							ipconfig->address->peer);
+	} else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4) {
+		return __connman_connection_gateway_add(service,
+						ipconfig->address->gateway,
+							NULL, ipconfig->address->peer);
+	}
+
+	return 0;
+}
+
+void __connman_ipconfig_gateway_remove(struct connman_ipconfig *ipconfig)
+{
+	struct connman_service *service;
+
+	DBG("");
+
+	service = __connman_service_lookup_from_index(ipconfig->index);
+	if (service != NULL)
+		__connman_connection_gateway_remove(service);
+}
+
+unsigned char __connman_ipconfig_get_prefixlen(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return 0;
+
+	return ipconfig->address->prefixlen;
+}
+
+void __connman_ipconfig_set_prefixlen(struct connman_ipconfig *ipconfig, unsigned char prefixlen)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	ipconfig->address->prefixlen = prefixlen;
+}
+
 static struct connman_ipconfig *create_ipv6config(int index)
 {
 	struct connman_ipconfig *ipv6config;
@@ -852,7 +1139,8 @@ static struct connman_ipconfig *create_ipv6config(int index)
 
 	ipv6config->index = index;
 	ipv6config->type = CONNMAN_IPCONFIG_TYPE_IPV6;
-	ipv6config->method = CONNMAN_IPCONFIG_METHOD_OFF;
+	ipv6config->method = CONNMAN_IPCONFIG_METHOD_AUTO;
+	ipv6config->ipv6_privacy_config = 0;
 
 	ipv6config->address = connman_ipaddress_alloc(AF_INET6);
 	if (ipv6config->address == NULL) {
@@ -915,6 +1203,9 @@ struct connman_ipconfig *connman_ipconfig_create(int index,
  */
 struct connman_ipconfig *connman_ipconfig_ref(struct connman_ipconfig *ipconfig)
 {
+	DBG("ipconfig %p refcount %d", ipconfig,
+				g_atomic_int_get(&ipconfig->refcount) + 1);
+
 	g_atomic_int_inc(&ipconfig->refcount);
 
 	return ipconfig;
@@ -928,8 +1219,13 @@ struct connman_ipconfig *connman_ipconfig_ref(struct connman_ipconfig *ipconfig)
  */
 void connman_ipconfig_unref(struct connman_ipconfig *ipconfig)
 {
-	if (ipconfig &&
-		g_atomic_int_dec_and_test(&ipconfig->refcount) == TRUE) {
+	if (ipconfig == NULL)
+		return;
+
+	DBG("ipconfig %p refcount %d", ipconfig,
+			g_atomic_int_get(&ipconfig->refcount) - 1);
+
+	if (g_atomic_int_dec_and_test(&ipconfig->refcount) == TRUE) {
 		__connman_ipconfig_disable(ipconfig);
 
 		connman_ipconfig_set_ops(ipconfig, NULL);
@@ -1048,70 +1344,17 @@ enum connman_ipconfig_method __connman_ipconfig_get_method(struct connman_ipconf
 	return ipconfig->method;
 }
 
-/**
- * connman_ipconfig_bind:
- * @ipconfig: ipconfig structure
- * @ipaddress: ipaddress structure
- *
- * Bind IP address details to configuration
- */
-void connman_ipconfig_bind(struct connman_ipconfig *ipconfig,
-					struct connman_ipaddress *ipaddress)
-{
-	struct connman_ipconfig *origin;
-
-	origin = ipconfig->origin ? ipconfig->origin : ipconfig;
-
-	connman_ipaddress_copy(origin->address, ipaddress);
-
-	connman_inet_set_address(origin->index, origin->address);
-}
-
-void __connman_ipconfig_set_element_ipv6_gateway(
-			struct connman_ipconfig *ipconfig,
-				struct connman_element *element)
-{
-	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
-		element->ipv6.gateway = ipconfig->address->gateway;
-}
-
-/*
- * FIXME: The element soulution should be removed in the future
- * Set IPv4 and IPv6 gateway
- */
-int __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig,
-						struct connman_element *parent)
-{
-	struct connman_element *connection;
-
-	connection = connman_element_create(NULL);
-
-	DBG("ipconfig %p", ipconfig);
-
-	connection->type  = CONNMAN_ELEMENT_TYPE_CONNECTION;
-	connection->index = ipconfig->index;
-
-	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
-		connection->ipv4.gateway = ipconfig->address->gateway;
-	else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
-		connection->ipv6.gateway = ipconfig->address->gateway;
-
-	if (connman_element_register(connection, parent) < 0)
-		connman_element_unref(connection);
-
-	return 0;
-}
-
-int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig)
+int __connman_ipconfig_address_add(struct connman_ipconfig *ipconfig)
 {
 	DBG("");
 
 	switch (ipconfig->method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		break;
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		break;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
 			return connman_inet_set_address(ipconfig->index,
@@ -1124,8 +1367,10 @@ int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig)
 	return 0;
 }
 
-int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig)
+int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig)
 {
+	int err;
+
 	DBG("");
 
 	if (ipconfig == NULL)
@@ -1136,18 +1381,25 @@ int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig)
 	switch (ipconfig->method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		break;
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		break;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
-			return connman_inet_clear_address(ipconfig->index,
+			err = connman_inet_clear_address(ipconfig->index,
 							ipconfig->address);
 		else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
-			return connman_inet_clear_ipv6_address(
+			err = connman_inet_clear_ipv6_address(
 						ipconfig->index,
 						ipconfig->address->local,
 						ipconfig->address->prefixlen);
+		else
+			err = -EINVAL;
+
+		connman_ipaddress_clear(ipconfig->address);
+
+		return err;
 	}
 
 	return 0;
@@ -1191,6 +1443,46 @@ const char *__connman_ipconfig_get_proxy_autoconfig(struct connman_ipconfig *ipc
 	return ipdevice->pac;
 }
 
+static void disable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	struct connman_ipdevice *ipdevice;
+
+	DBG("");
+
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	set_ipv6_state(ipdevice->ifname, FALSE);
+}
+
+static void enable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	struct connman_ipdevice *ipdevice;
+
+	DBG("");
+
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO)
+		set_ipv6_privacy(ipdevice->ifname,
+				ipconfig->ipv6_privacy_config);
+
+	set_ipv6_state(ipdevice->ifname, TRUE);
+}
+
+void __connman_ipconfig_disable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig == NULL || ipconfig->type != CONNMAN_IPCONFIG_TYPE_IPV6)
+		return;
+
+	disable_ipv6(ipconfig);
+}
+
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 {
 	struct connman_ipdevice *ipdevice;
@@ -1216,6 +1508,7 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 		if (ipdevice->config_ipv6 == ipconfig)
 			return -EALREADY;
 		type = CONNMAN_IPCONFIG_TYPE_IPV6;
+		enable_ipv6(ipconfig);
 	} else
 		return -EINVAL;
 
@@ -1299,6 +1592,10 @@ int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig)
 	if (ipdevice->config_ipv6 == ipconfig) {
 		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
 
+		if (ipdevice->config_ipv6->method ==
+						CONNMAN_IPCONFIG_METHOD_AUTO)
+			disable_ipv6(ipdevice->config_ipv6);
+
 		connman_ipaddress_clear(ipdevice->config_ipv6->system);
 		connman_ipconfig_unref(ipdevice->config_ipv6);
 		ipdevice->config_ipv6 = NULL;
@@ -1321,6 +1618,8 @@ const char *__connman_ipconfig_method2string(enum connman_ipconfig_method method
 		return "manual";
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
 		return "dhcp";
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		return "auto";
 	}
 
 	return NULL;
@@ -1336,10 +1635,36 @@ enum connman_ipconfig_method __connman_ipconfig_string2method(const char *method
 		return CONNMAN_IPCONFIG_METHOD_MANUAL;
 	else if (g_strcmp0(method, "dhcp") == 0)
 		return CONNMAN_IPCONFIG_METHOD_DHCP;
+	else if (g_strcmp0(method, "auto") == 0)
+		return CONNMAN_IPCONFIG_METHOD_AUTO;
 	else
 		return CONNMAN_IPCONFIG_METHOD_UNKNOWN;
 }
 
+static const char *privacy2string(int privacy)
+{
+	if (privacy <= 0)
+		return "disabled";
+	else if (privacy == 1)
+		return "enabled";
+	else if (privacy > 1)
+		return "prefered";
+
+	return "disabled";
+}
+
+static int string2privacy(const char *privacy)
+{
+	if (g_strcmp0(privacy, "disabled") == 0)
+		return 0;
+	else if (g_strcmp0(privacy, "enabled") == 0)
+		return 1;
+	else if (g_strcmp0(privacy, "prefered") == 0)
+		return 2;
+	else
+		return 0;
+}
+
 void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
@@ -1380,9 +1705,10 @@ void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 }
 
 void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
-							DBusMessageIter *iter)
+					DBusMessageIter *iter,
+					struct connman_ipconfig *ipconfig_ipv4)
 {
-	const char *str;
+	const char *str, *privacy;
 
 	DBG("");
 
@@ -1393,6 +1719,12 @@ void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 	if (str == NULL)
 		return;
 
+	if (ipconfig_ipv4 != NULL &&
+			ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO) {
+		if (__connman_6to4_check(ipconfig_ipv4) == 1)
+			str = "6to4";
+	}
+
 	connman_dbus_dict_append_basic(iter, "Method", DBUS_TYPE_STRING, &str);
 
 	if (ipconfig->system == NULL)
@@ -1409,12 +1741,16 @@ void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 	if (ipconfig->system->gateway != NULL)
 		connman_dbus_dict_append_basic(iter, "Gateway",
 				DBUS_TYPE_STRING, &ipconfig->system->gateway);
+
+	privacy = privacy2string(ipconfig->ipv6_privacy_config);
+	connman_dbus_dict_append_basic(iter, "Privacy",
+				DBUS_TYPE_STRING, &privacy);
 }
 
 void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
-	const char *str;
+	const char *str, *privacy;
 
 	DBG("");
 
@@ -1431,6 +1767,7 @@ void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 		return;
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
 		break;
 	}
 
@@ -1448,6 +1785,10 @@ void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 	if (ipconfig->address->gateway != NULL)
 		connman_dbus_dict_append_basic(iter, "Gateway",
 				DBUS_TYPE_STRING, &ipconfig->address->gateway);
+
+	privacy = privacy2string(ipconfig->ipv6_privacy_config);
+	connman_dbus_dict_append_basic(iter, "Privacy",
+				DBUS_TYPE_STRING, &privacy);
 }
 
 void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
@@ -1468,6 +1809,7 @@ void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
 	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
 		return;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		break;
@@ -1497,19 +1839,15 @@ void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
 }
 
 int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
-		enum connman_ipconfig_type type, DBusMessageIter *array)
+							DBusMessageIter *array)
 {
 	enum connman_ipconfig_method method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
 	const char *address = NULL, *netmask = NULL, *gateway = NULL,
-			*prefix_length_string = NULL;
-	int prefix_length = 0;
+		*prefix_length_string = NULL, *privacy_string = NULL;
+	int prefix_length = 0, privacy = 0;
 	DBusMessageIter dict;
 
-	DBG("ipconfig %p type %d", ipconfig, type);
-
-	if (type != CONNMAN_IPCONFIG_TYPE_IPV4 &&
-			type != CONNMAN_IPCONFIG_TYPE_IPV6)
-		return -EINVAL;
+	DBG("ipconfig %p", ipconfig);
 
 	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
 		return -EINVAL;
@@ -1565,37 +1903,60 @@ int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
 				return -EINVAL;
 
 			dbus_message_iter_get_basic(&entry, &gateway);
+		} else if (g_str_equal(key, "Privacy") == TRUE) {
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &privacy_string);
+			privacy = string2privacy(privacy_string);
 		}
 		dbus_message_iter_next(&dict);
 	}
 
-	DBG("method %d address %s netmask %s gateway %s prefix_length %d",
-			method, address, netmask, gateway, prefix_length);
+	DBG("method %d address %s netmask %s gateway %s prefix_length %d "
+		"privacy %s",
+		method, address, netmask, gateway, prefix_length,
+		privacy_string);
 
 	switch (method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
 		return -EINVAL;
 
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+		ipconfig->method = method;
+		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
+			disable_ipv6(ipconfig);
+		break;
+
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		if (ipconfig->type != CONNMAN_IPCONFIG_TYPE_IPV6)
+			return -EINVAL;
+
+		ipconfig->method = method;
+		if (privacy_string != NULL)
+			ipconfig->ipv6_privacy_config = privacy;
+		enable_ipv6(ipconfig);
+		break;
+
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		if (address == NULL)
 			return -EINVAL;
 
 		ipconfig->method = method;
 
-		if (type == CONNMAN_IPCONFIG_TYPE_IPV4)
+		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
 			connman_ipaddress_set_ipv4(ipconfig->address,
 						address, netmask, gateway);
 		else
 			return connman_ipaddress_set_ipv6(
 					ipconfig->address, address,
-						gateway, prefix_length);
+						prefix_length, gateway);
 		break;
 
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		if (ipconfig->method == method)
-			return 0;
+		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
+			return -EOPNOTSUPP;
 
 		ipconfig->method = method;
 		break;
@@ -1634,7 +1995,7 @@ void __connman_ipconfig_append_ethernet(struct connman_ipconfig *ipconfig,
 int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix)
 {
-	const char *method;
+	char *method;
 	char *key;
 
 	DBG("ipconfig %p identifier %s", ipconfig, identifier);
@@ -1652,6 +2013,23 @@ int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 	if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 		ipconfig->method = CONNMAN_IPCONFIG_METHOD_OFF;
 
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+		if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO ||
+			ipconfig->method == CONNMAN_IPCONFIG_METHOD_MANUAL) {
+			char *privacy;
+			char *pprefix = g_strdup_printf("%sprivacy", prefix);
+			privacy = g_key_file_get_string(keyfile, identifier,
+							pprefix, NULL);
+			ipconfig->ipv6_privacy_config = string2privacy(privacy);
+			g_free(pprefix);
+			g_free(privacy);
+
+			__connman_ipconfig_enable(ipconfig);
+			enable_ipv6(ipconfig);
+		}
+	}
+
+	g_free(method);
 	g_free(key);
 
 	key = g_strdup_printf("%snetmask_prefixlen", prefix);
@@ -1696,6 +2074,25 @@ int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 	g_key_file_set_string(keyfile, identifier, key, method);
 	g_free(key);
 
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+		const char *privacy;
+		privacy = privacy2string(ipconfig->ipv6_privacy_config);
+		key = g_strdup_printf("%sprivacy", prefix);
+		g_key_file_set_string(keyfile, identifier, key, privacy);
+		g_free(key);
+	}
+
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		return 0;
+	}
+
 	key = g_strdup_printf("%snetmask_prefixlen", prefix);
 	g_key_file_set_integer(keyfile, identifier,
 			key, ipconfig->address->prefixlen);
