@@ -32,7 +32,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 {
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
-	connman_bool_t offlinemode, tethering;
+	connman_bool_t offlinemode, sessionmode;
 	const char *str;
 
 	DBG("conn %p", conn);
@@ -50,8 +50,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_basic(&dict, "ActiveProfile",
 						DBUS_TYPE_OBJECT_PATH, &str);
 
-	connman_dbus_dict_append_array(&dict, "Profiles",
-			DBUS_TYPE_OBJECT_PATH, __connman_profile_list, NULL);
 	connman_dbus_dict_append_array(&dict, "Services",
 			DBUS_TYPE_OBJECT_PATH, __connman_service_list, NULL);
 	connman_dbus_dict_append_array(&dict, "Technologies",
@@ -65,10 +63,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_basic(&dict, "OfflineMode",
 					DBUS_TYPE_BOOLEAN, &offlinemode);
 
-	tethering = __connman_tethering_get_status();
-	connman_dbus_dict_append_basic(&dict, "Tethering",
-					DBUS_TYPE_BOOLEAN, &tethering);
-
 	connman_dbus_dict_append_array(&dict, "AvailableTechnologies",
 		DBUS_TYPE_STRING, __connman_notifier_list_registered, NULL);
 	connman_dbus_dict_append_array(&dict, "EnabledTechnologies",
@@ -86,6 +80,11 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_array(&dict, "EnabledDebugs",
 			DBUS_TYPE_STRING, __connman_debug_list_enabled, NULL);
 
+	sessionmode = __connman_session_mode();
+	connman_dbus_dict_append_basic(&dict, "SessionMode",
+					DBUS_TYPE_BOOLEAN,
+					&sessionmode);
+
 	connman_dbus_dict_close(&array, &dict);
 
 	return reply;
@@ -120,26 +119,21 @@ static DBusMessage *set_property(DBusConnection *conn,
 		__connman_profile_set_offlinemode(offlinemode, TRUE);
 
 		__connman_profile_save_default();
-	} else if (g_str_equal(name, "Tethering") == TRUE) {
-		connman_bool_t tethering;
-
-		if (type != DBUS_TYPE_BOOLEAN)
-			return __connman_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &tethering);
-
-		if (__connman_tethering_set_status(tethering) < 0)
-			return __connman_error_invalid_arguments(msg);
-
-		connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
-					CONNMAN_MANAGER_INTERFACE, "Tethering",
-						DBUS_TYPE_BOOLEAN, &tethering);
 	} else if (g_str_equal(name, "ActiveProfile") == TRUE) {
 		const char *str;
 
 		dbus_message_iter_get_basic(&value, &str);
 
 		return __connman_error_not_supported(msg);
+	} else if (g_str_equal(name, "SessionMode") == TRUE) {
+		connman_bool_t sessionmode;
+
+		if (type != DBUS_TYPE_BOOLEAN)
+			return __connman_error_invalid_arguments(msg);
+
+		dbus_message_iter_get_basic(&value, &sessionmode);
+
+		__connman_session_set_mode(sessionmode);
 	} else
 		return __connman_error_invalid_property(msg);
 
@@ -159,43 +153,6 @@ static DBusMessage *get_state(DBusConnection *conn,
 						DBUS_TYPE_INVALID);
 }
 
-static DBusMessage *create_profile(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	const char *name, *path;
-	int err;
-
-	DBG("conn %p", conn);
-
-	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
-							DBUS_TYPE_INVALID);
-
-	err = __connman_profile_create(name, &path);
-	if (err < 0)
-		return __connman_error_failed(msg, -err);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_OBJECT_PATH, &path,
-							DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *remove_profile(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	const char *path;
-	int err;
-
-	DBG("conn %p", conn);
-
-	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
-							DBUS_TYPE_INVALID);
-
-	err = __connman_profile_remove(path);
-	if (err < 0)
-		return __connman_error_failed(msg, -err);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
 static DBusMessage *remove_provider(DBusConnection *conn,
 				    DBusMessage *msg, void *data)
 {
@@ -460,6 +417,13 @@ static DBusMessage *connect_service(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (__connman_session_mode() == TRUE) {
+		connman_info("Session mode enabled: "
+				"direct service connect disabled");
+
+		return __connman_error_failed(msg, -EINVAL);
+	}
+
 	err = __connman_service_create_and_connect(msg);
 	if (err < 0) {
 		if (err == -EINPROGRESS) {
@@ -473,6 +437,19 @@ static DBusMessage *connect_service(DBusConnection *conn,
 	return NULL;
 }
 
+static DBusMessage *provision_service(DBusConnection *conn, DBusMessage *msg,
+					void *data)
+{
+	int err;
+
+	DBG("conn %p", conn);
+
+	err = __connman_service_provision(msg);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
+
+	return NULL;
+}
 
 static DBusMessage *connect_provider(DBusConnection *conn,
 					DBusMessage *msg, void *data)
@@ -481,6 +458,13 @@ static DBusMessage *connect_provider(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (__connman_session_mode() == TRUE) {
+		connman_info("Session mode enabled: "
+				"direct provider connect disabled");
+
+		return __connman_error_failed(msg, -EINVAL);
+	}
+
 	err = __connman_provider_create_and_connect(msg);
 	if (err < 0) {
 		if (err == -EINPROGRESS) {
@@ -579,40 +563,28 @@ static DBusMessage *unregister_counter(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static DBusMessage *request_session(DBusConnection *conn,
+static DBusMessage *create_session(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	const char *bearer, *sender, *service_path;
-	struct connman_service *service;
-
-	DBG("conn %p", conn);
-
-	sender = dbus_message_get_sender(msg);
-
-	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &bearer,
-							DBUS_TYPE_INVALID);
-
-	service = __connman_session_request(bearer, sender);
-	if (service == NULL)
-		return __connman_error_failed(msg, EINVAL);
-
-	service_path = __connman_service_get_path(service);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_OBJECT_PATH, &service_path,
-						DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *release_session(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	const char *sender;
 	int err;
 
 	DBG("conn %p", conn);
 
-	sender = dbus_message_get_sender(msg);
+	err = __connman_session_create(msg);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
 
-	err = __connman_session_release(sender);
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
+static DBusMessage *destroy_session(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	int err;
+
+	DBG("conn %p", conn);
+
+	err = __connman_session_destroy(msg);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
 
@@ -623,8 +595,6 @@ static GDBusMethodTable manager_methods[] = {
 	{ "GetProperties",     "",      "a{sv}", get_properties     },
 	{ "SetProperty",       "sv",    "",      set_property       },
 	{ "GetState",          "",      "s",     get_state          },
-	{ "CreateProfile",     "s",     "o",     create_profile     },
-	{ "RemoveProfile",     "o",     "",      remove_profile     },
 	{ "RemoveProvider",    "o",     "",      remove_provider    },
 	{ "RequestScan",       "s",     "",      request_scan       },
 	{ "EnableTechnology",  "s",     "",      enable_technology,
@@ -635,14 +605,16 @@ static GDBusMethodTable manager_methods[] = {
 	{ "LookupService",     "s",     "o",     lookup_service,    },
 	{ "ConnectService",    "a{sv}", "o",     connect_service,
 						G_DBUS_METHOD_FLAG_ASYNC },
+	{ "ProvisionService",  "s",     "",      provision_service,
+						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "ConnectProvider",   "a{sv}", "o",     connect_provider,
 						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "RegisterAgent",     "o",     "",      register_agent     },
 	{ "UnregisterAgent",   "o",     "",      unregister_agent   },
 	{ "RegisterCounter",   "ouu",   "",      register_counter   },
 	{ "UnregisterCounter", "o",     "",      unregister_counter },
-	{ "RequestSession",    "s",     "o",     request_session    },
-	{ "ReleaseSession",    "s",     "",      release_session    },
+	{ "CreateSession",     "a{sv}o", "o",    create_session     },
+	{ "DestroySession",    "o",     "",      destroy_session    },
 	{ },
 };
 
@@ -652,79 +624,7 @@ static GDBusSignalTable manager_signals[] = {
 	{ },
 };
 
-static DBusMessage *nm_sleep(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	DBusMessage *reply;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_append_args(reply, DBUS_TYPE_INVALID);
-
-	return reply;
-}
-
-static DBusMessage *nm_wake(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	DBusMessage *reply;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_append_args(reply, DBUS_TYPE_INVALID);
-
-	return reply;
-}
-
-enum {
-	NM_STATE_UNKNOWN = 0,
-	NM_STATE_ASLEEP,
-	NM_STATE_CONNECTING,
-	NM_STATE_CONNECTED,
-	NM_STATE_DISCONNECTED
-};
-
-static DBusMessage *nm_state(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	DBusMessage *reply;
-	dbus_uint32_t state;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	if (__connman_notifier_count_connected() > 0)
-		state = NM_STATE_CONNECTED;
-	else
-		state = NM_STATE_DISCONNECTED;
-
-	dbus_message_append_args(reply, DBUS_TYPE_UINT32, &state,
-							DBUS_TYPE_INVALID);
-
-	return reply;
-}
-
-static GDBusMethodTable nm_methods[] = {
-	{ "sleep", "",  "",   nm_sleep        },
-	{ "wake",  "",  "",   nm_wake         },
-	{ "state", "",  "u",  nm_state        },
-	{ },
-};
-
-static gboolean nm_compat = FALSE;
-
-int __connman_manager_init(gboolean compat)
+int __connman_manager_init(void)
 {
 	DBG("");
 
@@ -740,13 +640,6 @@ int __connman_manager_init(gboolean compat)
 					manager_methods,
 					manager_signals, NULL, NULL, NULL);
 
-	if (compat == TRUE) {
-		g_dbus_register_interface(connection, NM_PATH, NM_INTERFACE,
-					nm_methods, NULL, NULL, NULL, NULL);
-
-		nm_compat = TRUE;
-	}
-
 	return 0;
 }
 
@@ -759,10 +652,6 @@ void __connman_manager_cleanup(void)
 	if (connection == NULL)
 		return;
 
-	if (nm_compat == TRUE) {
-		g_dbus_unregister_interface(connection, NM_PATH, NM_INTERFACE);
-	}
-
 	g_dbus_unregister_interface(connection, CONNMAN_MANAGER_PATH,
 						CONNMAN_MANAGER_INTERFACE);
 
