@@ -214,6 +214,9 @@ static void stats_free(gpointer user_data)
 {
 	struct stats_file *file = user_data;
 
+	if (file == NULL)
+		return;
+
 	msync(file->addr, file->len, MS_SYNC);
 
 	munmap(file->addr, file->len);
@@ -222,9 +225,16 @@ static void stats_free(gpointer user_data)
 	TFR(close(file->fd));
 	file->fd = -1;
 
-	if (file->history_name != NULL)
+	if (file->history_name != NULL) {
 		g_free(file->history_name);
-	g_free(file->name);
+		file->history_name = NULL;
+	}
+
+	if (file->name != NULL) {
+		g_free(file->name);
+		file->name = NULL;
+	}
+
 	g_free(file);
 }
 
@@ -267,6 +277,9 @@ static int stats_file_remap(struct stats_file *file, size_t size)
 	void *addr;
 	int err;
 
+	DBG("file %p size %zu addr %p len %zu", file, size, file->addr,
+		file->len);
+
 	page_size = sysconf(_SC_PAGESIZE);
 	new_size = (size + page_size - 1) & ~(page_size - 1);
 
@@ -278,6 +291,13 @@ static int stats_file_remap(struct stats_file *file, size_t size)
 	}
 
 	if (file->addr == NULL) {
+		/*
+		 * Though the buffer is not shared between processes, we still
+		 * have to take MAP_SHARED because MAP_PRIVATE does not
+		 * guarantee that writes will hit the file without an explicit
+		 * call to munmap or msync. For more details please read the
+		 * mmap man pages.
+		 */
 		addr = mmap(NULL, new_size, PROT_READ | PROT_WRITE,
 				MAP_SHARED, file->fd, 0);
 	} else {
@@ -287,6 +307,11 @@ static int stats_file_remap(struct stats_file *file, size_t size)
 	if (addr == MAP_FAILED) {
 		connman_error("mmap error %s for %s",
 				strerror(errno), file->name);
+		if (errno == EINVAL) {
+			connman_error("%s might be on a file system, such as "
+					"JFFS2, that does not allow shared "
+					"writable mappings.", file->name);
+		}
 		return -errno;
 	}
 
@@ -301,6 +326,8 @@ static int stats_file_remap(struct stats_file *file, size_t size)
 static int stats_open(struct stats_file *file,
 			const char *name)
 {
+	DBG("file %p name %s", file, name);
+
 	file->name = g_strdup(name);
 
 	file->fd = TFR(open(file->name, O_RDWR | O_CREAT, 0644));
@@ -308,6 +335,7 @@ static int stats_open(struct stats_file *file,
 		connman_error("open error %s for %s",
 				strerror(errno), file->name);
 		g_free(file->name);
+		file->name = NULL;
 		return -errno;
 	}
 
@@ -323,6 +351,7 @@ static int stats_open_temp(struct stats_file *file)
 		connman_error("create tempory file error %s for %s",
 				strerror(errno), file->name);
 		g_free(file->name);
+		file->name = NULL;
 		return -errno;
 	}
 
@@ -336,6 +365,8 @@ static int stats_file_setup(struct stats_file *file)
 	size_t size = 0;
 	int err;
 
+	DBG("file %p fd %d name %s", file, file->fd, file->name);
+
 	err = fstat(file->fd, &st);
 	if (err < 0) {
 		connman_error("fstat error %s for %s\n",
@@ -343,6 +374,7 @@ static int stats_file_setup(struct stats_file *file)
 
 		TFR(close(file->fd));
 		g_free(file->name);
+		file->name = NULL;
 
 		return -errno;
 	}
@@ -357,6 +389,7 @@ static int stats_file_setup(struct stats_file *file)
 	if (err < 0) {
 		TFR(close(file->fd));
 		g_free(file->name);
+		file->name = NULL;
 
 		return err;
 	}
@@ -572,6 +605,7 @@ static void stats_file_cleanup(struct stats_file *file)
 {
 	file->fd = -1;
 	g_free(file->name);
+	file->name = NULL;
 }
 
 static int stats_file_close_swap(struct stats_file *history_file,
